<?php

/**
 Written by Demetrius Papademetriou (DemPap)
 demkpap@gmail.com
 Please feel free to copy, alter and distribute this code at your own responsibility
 */


//Make sure the Matrix class is included:
//This class was written by  Jon Lawrence.It was dowloaded from www.phpclasses.org

require_once "matrix.class.php";

/********/

function solve_eq3($w, $a,$l,$E,$I)
//Solve the equation of overhanging u.l.d. beam with three supports, one in the center and two at a distance $a from the ends
//$w is the load per unit length, $a the distance from ends of beam of the outer supports, $l the distance between these two supports
//$E the elasticity modulus of beam and $I the moment of inertia of beam     
//Returns an array of four values: the force upwards at each outer supports, the force in the middle support the coefficient  A
//and the coefficient  B, needed for the solution of the deflection equation:
//EIy=-(F/6)(x-a)^3/6-(G/6)(x-(l+2a)/2)^3-(F/6)(x-(l+a))^3+(w/24)^4 +Ax+B    
{         
	$coef=sprintf("[%f,%f,%f,%f;%f,%f,%f,%f;%f,%f,%f,%f;%f,%f,%f,%f]",0,0,$a,1,	-$l*$l*$l/8/6,0,($l/2+$a),1, 
																										-$l*$l*$l/6,-$l*$l*$l/8/6, $a+$l,1,   2,1,0,0); 
	$const=sprintf("[%f;%f;%f;%f]",-$w*$a*$a*$a*$a/24,-$w*($l/2+$a)*($l/2+$a)*($l/2+$a)*($l/2+$a)/24,-$w*($a+$l)*($a+$l)*($a+$l)*($a+$l)/24,$w*($l+2*$a));   
//	print "COEF: $coef<br/>";
//	print "CONST: $const<br/>";
//echo "COEF: $coef	<br/><br/>CONST: $const <br/>";
//exit;
	$mcoef = new LW_Matrix($coef);
	$ccoef=  new LW_Matrix($const); 
   $invA = $mcoef->inverse();
   /*         echo matrixTable($invA, "A<sup>-1</sup>C=");
   exit;*/
   $solution=$invA->mpMatrix($ccoef);
   $retarr=$solution->getArray();
   if ($retarr[1][0] <= 0) { //Middle force is negative so we have only 2 supporte at distance $a from ends
   	$retarr=solve_eq2($w, $a,$l,$E,$I);
   }
	return ($retarr);        
        
}

/*******/

function solve_eq2($w, $a,$l,$E,$I)
{
	$retarr[1][0]=0;		//Middle force
	$retarr[0][0]=$w*($l+2*$a)/2; 	//Side forces equal
	$retarr[2][0]=(4*$retarr[0][0]*pow($l,3)-$w*(pow($a+$l,4)-pow($a,4)))/(24*$l);
	
	$retarr[3][0]=-$w/24*pow($a,4)-$a*$retarr[2][0];
   return($retarr);   
}

/********/

function calc_load_parms($x, $w, $a,$l,$E,$I,$F,$G,$A,$B)
//Calculates at and returns an array of load parameters at distance $x from left end of beam
//The returned array contains the following parameters at point $x: Foce at the ends, force in the middle, deflection, 
//tangent of curve bending moment,shear force 
{     
$y= ($x-$a>0?-$F/6*pow($x-$a,3):0) + ($x-($l+2*$a)/2>0?-$G/6*pow($x-($l+2*$a)/2,3):0)+($x-($l+$a)>0?-$F/6*pow($x-($l+$a),3):0)  +$w/24*pow($x,4)+$A*$x+$B;
$y=$y/($E*$I);
$arc=($x-$a>0?-$F/2*pow($x-$a,2):0) + ($x-($l+2*$a)/2>0?-$G/2*pow($x-($l+2*$a)/2,2):0)+($x-($l+$a)>0?-$F/2*pow($x-($l+$a),2):0)  +$w/6*pow($x,3)+$A; 
$arc=$arc/($E*$I);
$M=($x-$a>0?-$F*($x-$a):0) + ($x-($l+2*$a)/2>0?-$G*($x-($l+2*$a)/2):0)+	($x-($l+$a)>0?-$F*($x-($l+$a)):0) +$w*pow($x,2)/2;
$SF= ($x-$a>0?-$F:0)+ ($x-($l+2*$a)/2>0?-$G:0)+($x-($l+$a)>0?-$F:0)  +$w*$x;
$UL=$w; //Not return this for the time being
return (array($y,$arc,$M,$SF));
}          

/*******/

//Create a function to turn a Matrix into a styled table           
function matrixTable(LW_Matrix $matrix, $cap)
{
    $mArray = $matrix->getArray();
    $out = "<div class=\"LW_Matrix_float\">\n  <table class=\"LW_Matrix_table\">\n";
    $out .= "    <caption class=\"LW_left\">$cap</caption>\n";
    foreach($mArray as $row) {
        $out .= "    <tr class=\"LW_Matrix_row\">\n";
        foreach($row as $col) {
                $col1 = sprintf("%.10f", $col);
/*            if(floor($col)!= $col) {
                $col1 = sprintf("%.8f", $col);
                if(stristr($col1, ".00")) {
                    //two decimal places didn't change the value, so don't have them
                    $col1 = sprintf("%.0f", $col);
                }
            } else {
                $col1 = $col;
            }*/
            $out .= "      <td class=\"LW_Matrix_col\">$col1</td>\n";
        }
        $out .= "    </tr>\n";
    }
    $out .= "  </table>\n</div>\n";
    return $out; 
}

/********/

function get_decimal_count($fl) 
{
/*	echo $fl*10;
for ($i=0;$i<10;$i++) {
	$fl=$fl*10;
echo "Number:[".((string)$fl)."] Floor:[".((string)floor($fl))."] Ceil: [".((string)ceil($fl))."]<br/>";
if((string)$fl===(string)floor($fl) || ((string)$fl===(string)ceil($fl))) echo "Is equal <br/>";
//echo (string)$f."<br/>";
	}
return;*/
//echo $fl;
$fl=$fl*1;
$num = 0;
	while (true) { 
		if((string)$fl===(string)floor($fl) || ((string)$fl===(string)ceil($fl))) {
			break;
		}
//	echo "Number:[".(string)$fl."] Floor:[".(string)floor($fl)."] Ceil: [".ceil($fl)."]<br/>";
		$fl *= 10;
		$num++;
//		echo "f:$fl<br/>";
	}   
return $num;
}

/********/                                                                                               

//next we'll add the boring html/css header stuff
?>

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>overhnaging beam with 3 supports</title>
<style type="text/css">
.LW_Matrix_table {
	border: 1px black solid;
	border-collapse: collapse;
	padding: 5px;
}
.LW_Matrix_row {
	padding: 5px;
}
.LW_Matrix_col {
	border: 1px black dashed;
	padding: 5px;
	text-align:center;
}
.LW_Matrix_float {
	float: left;
	margin: 10px;
	vertical-align: middle;
}
.LW_Id_float {
	text-align:center;
	vertical-align: middle;
	float: left;
}
caption.LW_left {
	caption-side: left;
	vertical-align: middle;
}
.clearFloat {
    clear: both;
}
div.LW_output {
    border: 1px black solid;
    overflow: auto;
    width:600px;
}
</style>
</head>

<body>

<?php

if (isset($_POST['doubleyou']) && isset($_POST['alpha']) && isset($_POST['el']) && isset($_POST['epsilon']) && 
					isset($_POST['iota']) && isset($_POST['intvl']) && isset($_POST['howmany'])) {
	$w = $_POST['doubleyou'];
	$a= $_POST['alpha'];
	$l= $_POST['el'];
	$E= $_POST['epsilon'];
	$I= $_POST['iota'];
	$intvl=$_POST['intvl'];
	$howmany=$_POST['howmany'];
}
else {
	$w = "";
	$a= "";
	$l= "";
	$E= "";
	$I= "";
	$intvl="";
	$howmany="3";
}		

//Further explanation in HTML + HTML Form
?>

<table>
<tr>
<td colspan="3" height="30" style="color:red;font-size:120%;text-align:center;vertical-align:top"><b><u>OVERHANGING BEAM WITH 3 OR 2 SYMMETRICAL SUPPORTS</u></b></td>
</tr>

<tr><td bgcolor=#ffffcc>
<form name="theForm" method="post" action="">
  <label>Enter distance [a]:</label>
  <input type="text" name="alpha" id="alpha" value="<?php echo $a; ?>" size="15">
  <label >length units</label>
  <br/>
  <label>Enter distance [l or 2*l/2]:</label>
  <input type="text" name="el" id="el" value="<?php echo $l; ?>" size="15">
  <label>length units</label>
  <br/>
  <label for="epsilon">Elasticity modulus:</label>
  <input type="text" name="epsilon" id="epsilon" value="<?php echo $E; ?>" size="15">
  <label>force/length<sup>2</sup> units</label>
  <br/>
  <label>Moment of inertia:</label>
  <input type="text" name="iota" id="iota" value="<?php echo $I; ?>" size="15">
  <label>length<sup>3</sup> units</label>
 <br/>
  <label>Uniformly distributed load:</label>
  <input type="text" name="doubleyou" id="doubleyou" value="<?php echo $w; ?>" size="10">
  <label>force/length units</label>
  <br/>
  <label>Calculation interval:</label>
  <input type="text" name="intvl" id="intvl" value="<?php echo $intvl; ?>" size="10">
  <label>length units</label>
  <br/>
  <label>How many supports:</label>
  <input type="radio" name="howmany" value="2" <?php if ($howmany=="2") echo "checked";?> >Two&nbsp;
  <input type="radio" name="howmany" value="3" <?php if ($howmany=="3") echo "checked";?> >Three<br/>
  <br/>
  <input type="submit" name="submit" id="submit" value="Calculate">

</form>

</td>
<td valign="middle"  bgcolor=#eeeeee>
<table align="center">
<tr><td width="65" height="30" align="center"><a href="info.pdf"  title="Short information about the program" target="_blank"><img src="info.jpg" width="50" border="0" /></a></td></tr>

<tr><td width="65" height="30" align="center"><a href="beam.zip"  title="Source code in php" target="_blank"><img src="code.jpg"  width="50" border="0" /></a></td></tr>
</table></td>
<td valign="middle" bgcolor=#aaaaaa>
<img src="load_picture.jpg" width="350">
</td></tr></table>
<?php

//Ok, form and everything displayed, now to do the math and display further output based on that.
//check to see if anything was submitted before trying to do the math:

if ( $a!="" && $l!="" &&  $E!="" && $I!="" && $w!="" && $intvl!="")     {
	$dec= max(get_decimal_count($intvl),get_decimal_count($a+2*$l));
	
	if ($howmany=="3") $solarr=solve_eq3($w, $a,$l,$E,$I);
	else $solarr=solve_eq2($w, $a,$l,$E,$I);
	
	$F=$solarr[0][0];
	$G=$solarr[1][0];
	$A=$solarr[2][0];
	$B=$solarr[3][0];
	echo '<br/><table border="1">';
	echo '<tr align="center"><th>&nbsp;&nbsp;Lateral force, F&nbsp;&nbsp; </th><th>&nbsp;&nbsp;Central Force, G&nbsp;&nbsp;</th><th>&nbsp;&nbsp;A coeff&nbsp;&nbsp;</th><th>&nbsp;&nbsp;B coeff&nbsp;&nbsp;</th></tr>'; 
	echo '<tr align="center"><td>'.$F.'</td><td>'.$G.'</td><td>'.$A.'</td><td>'.$B.'</td></tr>'; 
	echo '</table><br/>';
	
	echo '<table border="1">';
	echo '<tr><th>X</th><th>Deflection</th><th>tan(θ)</th><th>Moment</th><th>Shear force</th></tr>'; 
	for ($x=0;$x<=$l+2*$a;$x+=round($intvl,$dec)) {

		$sol=calc_load_parms($x, $w, $a,$l,$E,$I,$F,$G,$A,$B) ;
		echo '<tr><td>'.round($x,$dec).'</td><td>'.$sol[0].'</td><td>'.$sol[1].'</td><td>'.$sol[2].'</td><td>'.$sol[3].'</td></tr>'; 
/*		echo "<b>X: $x</b><br/>";
		print_r($sol);
		echo "<br/>";*/
	}
	if( round($l+2*$a,$dec) != $x-round($intvl,$dec)) {
		$sol=calc_load_parms($l+2*$a, $w, $a,$l,$E,$I,$F,$G,$A,$B) ;
		echo '<tr><td>'.round($l+2*$a,$dec).'</td><td>'.$sol[0].'</td><td>'.$sol[1].'</td><td>'.$sol[2].'</td><td>'.$sol[3].'</td></tr>'; 
	}

	echo '</table>';
}


/*                          

if($coeff!="" && $consts!="")
{
    //try/catch to we don't get a fatal error
    try {
        $A = new LW_Matrix($coeff);
        $C = new LW_Matrix($consts);
        
        //Get the inverse of A
        $invA = $A->inverse();
        
        //Multiply the inverse of A by C to get the solution set
        $sols = $invA->mpMatrix($C);
        //Now we'll display all 3 matrices so user can check input
        echo "<p>Solution:</p>";
        echo matrixTable($A, "A=");
        echo matrixTable($C, "C=");
        echo matrixTable($invA, "A<sup>-1</sup>=");
        //clear the floats
        echo "<div class=\"clearFloat\">&nbsp;</div>\n";
        //now display the Solution with Caption for the Multiplication preformed
        echo matrixTable($sols, "A<sup>-1</sup>C=");
    } catch (Exception $e) {
        echo "<p>Error:<br />". $e->getMessage() ."</p>";
        //do nothing for now
    }
}
*/
//Finish the HTML to complete the page now...
?>
</body>
</html>